/*

Tkwav

Copyright (c) 1998, Nat Pryce

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in
all copies of this software.

IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, 
SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF 
THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE AUTHOR HAS BEEN ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" 
BASIS, AND THE AUTHOR HAS NO OBLIGATION TO PROVIDE  MAINTENANCE, SUPPORT,
UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

*/
#ifndef _AUTOCLOSE_H
#define _AUTOCLOSE_H



/*  Automagically closes a handle (or releases a resource) when destroyed.
 *  Useful for making functions exception safe.
 */
template <typename T, typename CLOSE_FN>
class AutoClose
{
public:
	AutoClose( const T &handle, const CLOSE_FN &close = CLOSE_FN() ) : 
		_handle(handle), _close(close) {
	}
	
	operator const T& () const { return _handle; }

	const AutoClose &operator = ( const T &handle ) {
		_close(_handle);
		_handle = handle;
		return *this;
	}
	
	~AutoClose() {
		_close(_handle);
	}
	
private:
	T _handle;
	CLOSE_FN _close;
};




#endif // _AUTOCLOSE_H

