/*

Tkwav

Copyright (c) 1998, Nat Pryce

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in
all copies of this software.

IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, 
SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF 
THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE AUTHOR HAS BEEN ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" 
BASIS, AND THE AUTHOR HAS NO OBLIGATION TO PROVIDE  MAINTENANCE, SUPPORT,
UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

*/
#ifndef _HASH_H
#define _HASH_H

#include <string>
#include <tcl.h>


using std::string;


template <class T>
class Hashtable
{
public:
	Hashtable();
	
	T *put( string key, T *data );
	T *put( const char *key, T *data );

	T *find( string key ) const;
	T *find( const char *key ) const;

	T *remove( string key );
	T *remove( const char *key );
	
	~Hashtable();

private:
	Tcl_HashTable _table;
};

template <class T> inline
Hashtable<T>::Hashtable() {
	Tcl_InitHashTable( &_table, TCL_STRING_KEYS );
}

template <class T> inline
T *Hashtable<T>::put( string key, T *data ) {
	return put( key.c_str(), data );
}

template <class T>
T *Hashtable<T>::put( const char *key, T *data ) {
	int is_new = 0;
	Tcl_HashEntry *e = Tcl_CreateHashEntry( &_table, key, &is_new );
	
	T *result = is_new ? reinterpret_cast<T*>(Tcl_GetHashValue(e)) : 0;

	Tcl_SetHashValue( e, reinterpret_cast<ClientData>(data) );

	return result;
}

template <class T> inline
T *Hashtable<T>::find( string key ) const {
	return find(key.c_str());
}

template <class T>
T *Hashtable<T>::find( const char *key ) const {
	Tcl_HashEntry *e = 
		Tcl_FindHashEntry( const_cast<Tcl_HashTable*>(&_table), key );

	if( e ) {
		return reinterpret_cast<T*>(Tcl_GetHashValue(e));
	} else {
		return 0;
	}
}


template <class T> inline
T *Hashtable<T>::remove( string key ) {
	return remove(key.c_str());
}

template <class T>
T *Hashtable<T>::remove( const char *key ) {
	Tcl_HashEntry *e = Tcl_FindHashEntry( &_table, key );
	if( e ) {
		T *result = reinterpret_cast<T*>(Tcl_GetHashValue(e));
		Tcl_DeleteHashEntry(e);
		return result;
	} else {
		return 0;
	}
}

template <class T>
Hashtable<T>::~Hashtable() {
	Tcl_DeleteHashTable( &_table );
}


#endif // _HASH_H
