

array set install [list \
	package	"Tkwav" \
	dirname tkwav \
	files	[list tkwav.dll pkgIndex.tcl MSVCP50.DLL] \
	libdir	[file dirname $tcl_library] \
]


proc startInstall {to} {
	global install
	
	if {![file exists $to]} {
		file mkdir $to
		.info configure -text "Created directory $to"
	} elseif {![file isdirectory $to]} {
		tk_messageBox -title "Error" -type ok \
			-message "$to is not a directory.\nSelect a directory and try again"
		return
	}
	
	set from [file dirname [info script]]
	
	set i 0
	foreach f $install(files) {
		set source [file join $from $f]
		set dest [file join $to $f]
		
		if {[file exists $dest]} {
			set choice [tk_messageBox -title "Overwrite File?" \
				-message "File $dest already exists.\nDo you want to overwrite it?" \
				-type yesno -default yes -icon warning]
			if {"$choice" == "no"} continue
		}
		file copy -force -- $source $dest
		.info configure -text "Copied $source to $dest"
		.files selection set 0 $i
		.files see $i
		update
		incr i
	}

	.info configure -text "Installation complete"

	destroy .install .cancel
	button .exit -text "Exit" -width 8 -command exit
	pack .exit -in .buttons -side bottom -padx 8 -pady 8
}

wm title . "Install $install(package)"


# A scrollable listbox showing the files to be installed.

frame .f1
listbox .files -relief sunken -borderwidth 2 -exportselection no \
	-xscrollcommand {.f1.xscroll set} \
	-yscrollcommand {.f1.yscroll set}
bindtags .files .files
scrollbar .f1.xscroll -orient horizontal -command {.files xview}
scrollbar .f1.yscroll -orient vertical -command {.files yview}
grid configure .files -in .f1 -row 0 -column 0 -sticky nsew -padx 2 -pady 2
grid configure .f1.xscroll -row 1 -column 0 -sticky nsew -padx 2 -pady 2
grid configure .f1.yscroll -row 0 -column 1 -sticky nsew -padx 2 -pady 2
grid rowconfigure .f1 0 -weight 1
grid columnconfigure .f1 0 -weight 1

foreach f $install(files) {
	.files insert end $f
}

# Entry containing the directory in which files are to be installed

label .dirL -text "Install to:"
entry .dir -width 60
.dir insert 0 [file join $install(libdir) $install(dirname)]

# Status bar

label .infoL -text "Information:"
label .info -relief sunken -borderwidth 1

# Ok/Cancel buttons

frame .buttons
button .install -text "Install" -width 8 -command {startInstall [.dir get]}
button .cancel -text "Cancel" -width 8 -command exit

# Lay out the widgets...

grid configure .f1 -row 0 -column 0 -columnspan 2 -padx 0 -pady 0 -sticky nsew
grid configure .dirL -row 1 -column 0 -padx 2 -pady 2 -sticky nsew
grid configure .dir -row 1 -column 1 -padx 2 -pady 2 -sticky nsew
grid configure .infoL -row 2 -column 0 -padx 2 -pady 2 -sticky nsew
grid configure .info -row 2 -column 1 -padx 2 -pady 2 -sticky nsew
grid configure .buttons -row 3 -column 0 -columnspan 2 -padx 2 -pady 2 -sticky sew

grid configure .install -in .buttons -row 0 -column 0 -padx 8 -pady 8 -sticky s
grid configure .cancel -in .buttons -row 0 -column 1 -padx 8 -pady 8 -sticky s
grid columnconfigure .buttons 0 -weight 1
grid columnconfigure .buttons 1 -weight 1

grid rowconfigure . 0 -weight 1
grid columnconfigure . 1 -weight 1


